/* 	Mayara Felix
	
	Goal: 	This code runs balance checks using simulated offers, using:
	
	1) true propensity scores ("simulated" files)
	2) first 1000 simulated offers
	
	For each balance regression, the code stores:
		1) the coefficient
		2) the t-stat on each coefficient
*/

clear all
cap log close
set more off
set trace off
set maxvar 32767
set matsize 11000
set seed 42
adopath + "Z:\ado\plus\"

pause on

local dta 		"Y:/DPS/MDmeetsRD Denver/Data/simulated_offers/"
local tables	"Y:\DPS\MDmeetsRD Denver\Tables\stata output\year2/"


local pscore_setup = 0
local pscore_balance = 0
local append_all = 0
local average_balance = 1
local market_size = 0
local erase_files = 0

local masterdate 		20160420 			/*20150708 20150603 20150525 20150521 Date of analysis file (an input to this code)*/
local outputdate 		20160420

local appcov "num_schools num_study rank1_study"
local cov 	"baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"
local cov2	"baseliness_math baseliness_reading baseliness_writing"

local attrition_vars 		"has_sc enrolls_next"

********************* Balance for each simulation *********************
forval sims=1/400{
		
		if `pscore_setup'==1{
				
			************************************************
			**************** Pscore Setup ******************
			************************************************		
				foreach file in simulation frequency formula{
					
						*Master dataset - with offers for each simulation
						use "`dta'mdrd_denver_`file'_master_year_2 `masterdate'", clear
						
						************************************************
						********* Determine which simulation ***********
						************************************************
						
						gen _offer_study = (offer`sims'==1) if appstudy==1				
						ren offer`sims' 		_offer
						
						drop offer*
						
						renvars _offer _offer_study, predrop(1)
						
						//Frequency and formula files have pscore as pscore_charter_frequency1, while 
						// simulation has as pscore_study_simulation. Renaming freq and formula as 
						// pscore_study_frequency or formula for consistency
						if "`file'"!="simulation"{
							ren pscore_charter_`file'`sims' _pscore_study_`file'
							drop pscore*
							
							renvars _pscore_study_`file', predrop(1)
						}
						
						
						************************************************
						
						/*Simulation pscores have huge support. Use rounded values for full saturation
						else if "`file'"=="simulation"{
							ren pscore_charter_`file' sim_pscore
							gen pscore_charter_`file' = round(sim_pscore,0.0001)
						}
						*/
						
						destring baselinegrade, replace
						destring appgrade, gen(appgrade_temp)
						drop appgrade
						rename appgrade_temp appgrade
						
							
						*Analysis at the student level. Generate max for charter offer
						bys maskedid: egen any_study_offer 	= max(offer_study)
						bys maskedid: egen num_schools = count(appschoolnum)
						bys maskedid: egen num_study = count(cond(appstudy==1,appschoolnum,.))
						gen pct_study = num_study/num_schools
						
						*Enrolls at charter flag (student level)
						bys maskedid: egen enroll_any_study 	= max(study)
						
						*Rank 1 at school level
						gen rank1_study= (appstudy==1 & apprank==1)
						
						*First ranked school is charter (student level, for cov balance)
						bys maskedid: egen temp = max(rank1_study)
						drop rank1_study
						ren temp rank1_study
						
						*Flag - apply to any study
						bys maskedid: egen max = max(appstudy)
						drop appstudy
						ren max appstudy
						
						*Enrolls next year (i.e. has nonmissing schoolnum)
						gen next = (schoolnum!=.)
						bys maskedid: egen enrolls_next = max(next)
						
						*** Dataset at student level from here on ****
						duplicates drop maskedid, force
						drop next
						
						*Redefine has scores so that it excludes sciences - very few kids
						replace has_sc = (has_sc_math | has_sc_reading | has_sc_writing)
						
						*Generate application school-grade dummies
						*egen sg = group(appschoolnum appgrade)
						*tab sg, gen(sg_)
						*School-level obs
						drop appschoolnum appschoolname offer appcharter apprank study charter
						
						*Keep only if in sample
					*keep if in_sample==1
					keep if has_pscore == 1 & appstudy == 1

					/*If simulation, replace back pscore with full support (just for bins)
					if "`file'"=="simulation"{
						replace pscore_charter_simulation = sim_pscore
					}
					*/
					
					** Rounded scores **	
					*Confirm they are mutually exclusive
						
						*Define r01 as meaning pscore rounded to 0.01
						gen r01 = round(pscore_study_`file', 0.01)
						tab r01, gen(r01_)
						
						*Define r001 as meaning pscore rounded to 0.001
						gen r001 = round(pscore_study_`file', 0.001)
						tab r001, gen(r001_)

					
					*Polynomials
					forvalues i=1/4{
						gen double pscore`i' = pscore_study_`file'^(`i')
					}
					
					//Updating for year 2 
					keep if baselinespyear==2013
			
					*Treatment is any study, instrument is study offer, riskset is pscore
					ren any_study_offer 	Z
					ren enroll_any_study  	D
					
					**********************************************************************************
					******************** Flag kids who are ineligble and guaranteed ******************
					**********************************************************************************
					*Flag kids who are ineligble and guaranteed - do not drop them, we still want them 
					*for OLS regressions and naive 2SLS (i.e. not controlling for risksets)
					
						*If simulation, first define in_zero1_pscore based on true score, then
						*round for true saturation (to tens of thousands)				
						if "`file'"=="simulation"{
							
							ren pscore_study_`file' sim_pscore
							gen in_zero1_pscore = (sim_pscore > 0 & sim_pscore < 1)
							gen pscore_charter_`file' = round(sim_pscore,0.0001)
							
							tab pscore_charter_`file', gen(r0_)			
							ren pscore_charter_`file' r0
						}
						*Else use actual score to both drop kids and create saturated risksets
						else{
											
							tab pscore_study_`file', gen(r0_)				
							ren pscore_study_`file' r0
							
							gen in_zero1_pscore = (r0 > 0 & r0 < 1)
						
						}

					**********************************************************************************
					********** Flag cells with variation in instrument and characteristics************
					**********************************************************************************
							
					/* 	Flag observations that actually make it into balance (i.e. obs in cells with
						variation in instrument). These flags are used for all types 
						of F-tests, conducted below.
						
						We generate these flags here so that no re-sorting of the data (done in bys x egen:)
						is done between the balanceregs and F tests, which affects how the variable
						"var", which stores the names of the variables corresponding to each balance
						coefficient, appears in the final dataset once we svmat the balance output.
					*/
					
						foreach x in r01 r001 r0{
							
							*Variation in instruments for each cell
								bys `x': egen sd_z_`x'	= 	sd(Z)
							
							*Good cell
								gen in_good_cell_`x' = (sd_z_`x'>0 & !missing(sd_z_`x') & in_zero1_pscore==1)
							
							gen double tag_`x' = `x'
							replace tag_`x' = -9999999 if in_good_cell_`x'==0
							tab tag_`x', gen(tag_`x'_)
							
							*Drop var for things with no var (the first one). These observations will not be included
							*in bin-based regression and fully saturated regressios. So, do not include their corresponding
							*variables either
							
							drop tag_`x'_1
							
							*Obs per cell
							
							bys `x': egen obs_cell_`x' = count(maskedid)
						}
			
					drop sd_z_*		
					save "`dta'analysis_`file'_year_2.dta", replace
					

					}/*Close file loop*/		
				}	/*Close pscore setup condition*/	
			
		
		*****************************************
		******** Balance and attrition **********
		*****************************************
		
		if `pscore_balance'==1{
			
		foreach file in simulation frequency formula{
			
				*Redefine the matrix at each loop
				matrix balance        = J(500,20,.)
			
				u "`dta'analysis_`file'_year_2", clear
		
				local row = 1
				local col = 1
				gen var =""
				
				
				*Count points in the support
				replace var = "Riskset points of support" if _n == 1
				
				
				*Place it on top of balance table
				unique pscore1 		if in_zero1_pscore==1
				matrix balance[1,3] = r(sum)
					
				unique r01			if in_good_cell_r01==1
				matrix balance[1,5] = r(sum)
					
				unique r001			if in_good_cell_r001==1
				matrix balance[1,6] = r(sum)
					
				unique r0			if in_good_cell_r0==1
				matrix balance[1,7] = r(sum)
				
				
				local row = `row' + 1
				
				foreach var in `appcov' `cov' `cov2' `attrition_vars'{
					replace var="`var'" if _n==`row'
					replace var="`var'_se" if _n==`row'+1
					replace var="`var'_N" if _n==`row'+2
				
					*All non-offered applicants
					qui summ `var' if Z==0
						matrix balance[`row',`col']   = r(mean)
						matrix balance[`row'+1,`col'] = r(sd)
						matrix balance[`row'+2,`col'] = r(N)
				
					*Balance and attrition, no control(OLS)
					qui reg `var' Z, r
						matrix balance[`row',`col'+1]   = _b[Z]
						matrix balance[`row'+1,`col'+1] = _se[Z]
						matrix balance[`row'+2,`col'+1] = e(N)
					
					*Balance and attrition, pscore linear control(OLS)
					qui reg `var' Z pscore1 if in_zero1_pscore==1, r
						matrix balance[`row',`col'+2]   = _b[Z]
						matrix balance[`row'+1,`col'+2] = _se[Z]
						matrix balance[`row'+2,`col'+2] = e(N)
						
					*Balance and attrition, pscore polynomial control(OLS)
					qui reg `var' Z pscore1 pscore2 pscore3 pscore4 if in_zero1_pscore==1, r
						matrix balance[`row',`col'+3]   = _b[Z]
						matrix balance[`row'+1,`col'+3] = _se[Z]
						matrix balance[`row'+2,`col'+3] = e(N)
							
					*Balance and attrition, risks
					local c = 0
					foreach i in 01 001{
						local c = `c' +1
						qui areg `var' Z if in_zero1_pscore==1, absorb(r`i') r
							matrix balance[`row',`col'+3 + `c']   = _b[Z]
							matrix balance[`row'+1,`col'+3 + `c'] = _se[Z]
							matrix balance[`row'+2,`col'+3 + `c'] = e(N)

					}
					
					*Fully staturated in pscore
					qui areg `var' Z if in_zero1_pscore==1, absorb(r0) r
						matrix balance[`row',`col'+4 + `c']   = _b[Z]
						matrix balance[`row'+1,`col'+4 + `c'] = _se[Z]
						matrix balance[`row'+2,`col'+4 + `c'] = e(N)
						
					local row = `row' + 4
					
				} /*Close variables loop*/
				
				******************************************************
				*************** Store these values *******************
				******************************************************
				
				svmat balance
				keep var balance*
				keep if _n<=`row'+5
				gen ID = "`file'_sim`sims'"
				ren balance1 z0
				ren balance2 ols_z1
				ren balance3 ols_z1_pscore
				ren balance4 ols_z1_poly
				ren balance5 ols_z1_r01
				ren balance6 ols_z1_r001
				ren balance7 ols_z1_psat
				order ID, first
				
				save "`dta'balance_`file'_sim_year_2_`sims'.dta", replace
				
				clear all
		
				} /*Close file loop*/
			} /*Close balance condition*/
	
		} /*Close simulation loop*/

********************* Average coefficients, and append all *********************
if `append_all'==1{

	foreach file in simulation frequency formula{
		** Zero1 master dataset ***

			u "`dta'balance_`file'_sim_year_2_1", replace
			forval sims=2/400{	
				append using "`dta'balance_`file'_sim_year_2_`sims'"
			}
			
			drop if var==""
			drop balance*
			
			save "`dta'balance_`file'_master_year_2 `outputdate'", replace

	}
}

if `average_balance'==1{
	
		foreach file in simulation frequency formula{
		** Generate average coefficient **
			
			u "`dta'balance_`file'_master_year_2 `outputdate'", clear
			keep if strpos(var,"_se")==0 & strpos(var,"_N")==0
			
			/*
			IR: leftover from year 1 
			** Appending mistake. sim1 appended twice...
			bys ID var: gen count = _n
			drop if count == 2
			*/
			
			foreach var of varlist z0-ols_z1_psat{
				bys var: egen mean_`var' = mean(cond(strpos(var,"support")==0,`var',.))
				bys var: egen mean_support_`var' = mean(cond(strpos(var,"support") > 0,`var',.))
			}
			
			keep var mean*
			duplicates drop
	
			
			outsheet using "`tables'average_`file'_coefficients_year_2 `outputdate'.csv", comma replace
		}
	}

	if `market_size' == 1{
	
		foreach file in simulation frequency formula{
		
			u "`dta'balance_`file'_master_year_2 `outputdate'", clear
			keep if index(var,"num_schools_N") > 0
			
			** Appending mistake. sim1 appended twice...
			bys ID var: gen count = _n
			drop if count == 2	
			
			foreach var of varlist z0-ols_z1_psat{
				bys var: egen mean_`var' = mean(`var')
			}
			keep var mean*
			duplicates drop
		
			outsheet using "`tables'average_market_size_`file'_year_2 `outputdate'.csv", comma replace
		}
	}
			
********************* Erase files *********************

if `erase_files'==1{
	foreach file in simulation frequency formula{
				cap erase "`dta'analysis_`file'_year_2.dta"
			forval sims=1/400{
				cap erase "`dta'balance_`file'_sim_year_2_`sims'.dta"
				
			}
	}
}
